//
//  igViewController.m
//  FacialDetection_ObjC
//
//  Created by Torrey Betts on 10/14/13.
//  Copyright (c) 2013 Infragistics. All rights reserved.
//

#import <IG/IG.h>
#import "igViewController.h"
#import "igSalesmanItem.h"

@interface igViewController () <IGGridViewDelegate>
{
    UIImageView *_imageView;
    UILabel *_detectionLabel;
    IGGridView *_gridView;
    NSArray *_data;
    IGGridViewSingleRowSingleFieldDataSourceHelper *_ds;
    IGGridViewLightTheme *_lightTheme;

    CGRect _bottomRect, _topRect;
}
@end

@implementation igViewController

- (void)viewDidLoad
{
    [super viewDidLoad];

    CGRectDivide(self.view.bounds, &_bottomRect, &_topRect, 200, CGRectMaxYEdge);

    [self initializeGrid];
    [self initializeMainContent];
}

-(void)initializeGrid
{
    _data = [igSalesmanItem generateData:50];

    _gridView = [[IGGridView alloc]initWithFrame:_bottomRect style:IGGridViewStyleDefault];
    _gridView.autoresizingMask = UIViewAutoresizingFlexibleWidth|UIViewAutoresizingFlexibleBottomMargin|UIViewAutoresizingFlexibleTopMargin;
    _gridView.selectionType = IGGridViewSelectionTypeCell;
    _gridView.headerHeight = 0;
    _gridView.emptyRows = NO;
    _gridView.rowSeparatorHeight = 0;
    _gridView.allowHorizontalBounce = YES;
    _gridView.alwaysBounceVertical = NO;
    _gridView.rowHeight = 200;
    _gridView.columnWidth = [[IGColumnWidth alloc]initWithWidth:200];
    _gridView.delegate = self;
    [self.view addSubview:_gridView];
    _gridView.theme = _lightTheme = [[IGGridViewLightTheme alloc] init];

    IGGridViewImageColumnDefinition* col = [[IGGridViewImageColumnDefinition alloc]initWithKey:@"image" forPropertyType:IGGridViewImageColumnDefinitionPropertyTypeImage];
    col.contentMode = UIViewContentModeScaleToFill;

    _ds = [[IGGridViewSingleRowSingleFieldDataSourceHelper alloc]initWithField:col];
    _ds.data  = _data;

    _gridView.dataSource = _ds;
}

-(void)initializeMainContent
{
    _imageView = [[UIImageView alloc] initWithFrame:CGRectInset(_topRect, 20, 20)];
    _imageView.autoresizingMask = UIViewAutoresizingFlexibleHeight|UIViewAutoresizingFlexibleWidth;
    _imageView.contentMode = UIViewContentModeScaleAspectFit;
    [self.view addSubview:_imageView];

    _detectionLabel = [[UILabel alloc] init];
    _detectionLabel.frame = CGRectMake(5, 5, 250, 100);
    _detectionLabel.autoresizingMask = UIViewAutoresizingFlexibleWidth|UIViewAutoresizingFlexibleHeight|UIViewAutoresizingFlexibleRightMargin;
    _detectionLabel.backgroundColor = [UIColor colorWithWhite:0.0 alpha:0.9];
    _detectionLabel.textColor = [UIColor whiteColor];
    _detectionLabel.textAlignment = NSTextAlignmentLeft;
    _detectionLabel.adjustsFontSizeToFitWidth = YES;
    _detectionLabel.numberOfLines = 0;
    [_imageView addSubview:_detectionLabel];

    IGCellPath *path = [[IGCellPath alloc] initForRow:0 inSection:0 inColumn:1];
    [_gridView selectCellAtPath:path animated:NO scrollPosition:IGGridViewScrollPositionTopLeft];

    IGGridViewImageCell *imageCell = [_gridView cellAtPath:path];
    _imageView.image = imageCell.imageView.image;
    [self facialDetection];
}

-(void)facialDetection
{
    int exifOrientation = 0;
    NSMutableString *detectionString = [[NSMutableString alloc] init];

    switch (_imageView.image.imageOrientation) {
        case UIImageOrientationUp:
            exifOrientation = 1;
            break;
        case UIImageOrientationDown:
            exifOrientation = 3;
            break;
        case UIImageOrientationLeft:
            exifOrientation = 8;
            break;
        case UIImageOrientationRight:
            exifOrientation = 6;
            break;
        case UIImageOrientationUpMirrored:
            exifOrientation = 2;
            break;
        case UIImageOrientationDownMirrored:
            exifOrientation = 4;
            break;
        case UIImageOrientationLeftMirrored:
            exifOrientation = 5;
            break;
        case UIImageOrientationRightMirrored:
            exifOrientation = 7;
            break;
        default:
            break;
    }

    NSDictionary *detectorOptions = @{ CIDetectorAccuracy : CIDetectorAccuracyHigh };
    CIDetector *faceDetector = [CIDetector detectorOfType:CIDetectorTypeFace context:nil options:detectorOptions];

    NSArray *features = [faceDetector featuresInImage:[CIImage imageWithCGImage:_imageView.image.CGImage]
                                              options:@{ CIDetectorSmile : @YES,
                                                         CIDetectorEyeBlink : @YES,
                                                         CIDetectorImageOrientation :[NSNumber numberWithInt:exifOrientation] }];

    for(CIFaceFeature *faceFeature in features)
    {
        NSString *hasSmile = faceFeature.hasSmile ? @"Yes" : @"No";
        NSString *hasLeftEye = faceFeature.hasLeftEyePosition ? @"Yes" : @"No";
        NSString *hasRightEye = faceFeature.hasRightEyePosition ? @"Yes" : @"No";
        NSString *hasLeftEyeBlink = faceFeature.leftEyeClosed ? @"Yes" : @"No";
        NSString *hasRightEyeBlink = faceFeature.rightEyeClosed ? @"Yes" : @"No";
        [detectionString appendFormat:@" SMILING: %@\n LEFT EYE: %@\n LEFT EYE BLINKING: %@\n RIGHT EYE: %@\n RIGHT EYE BLINKING: %@",
                        hasSmile, hasLeftEye, hasLeftEyeBlink, hasRightEye, hasRightEyeBlink];
    }

    _detectionLabel.text = detectionString;
}

- (void)gridView:(IGGridView *)gridView didSelectCellAtPath:(IGCellPath *)path
{
    IGGridViewImageCell *imageCell = [gridView cellAtPath:path];
    _imageView.image = imageCell.imageView.image;
    [self facialDetection];
}

@end